/*
 * Copyright 2012 Sebastian Mihai
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * audio.h
 *
 * 	Routines which provide support for sound effects and music. The audio.h/.c
 * 	module opens, closes, and maintains streams to the multimedia renderer (mmr)
 *
 *  Created on: 2012-08-09
 *      Author: Sebastian Mihai
 */

#ifndef AUDIO_H_
#define AUDIO_H_

// Background music
#define BGM_SOUTHERN 100

// Sound effects
#define SFX_TOUCH_CLICK 0
#define SFX_BAA_1 1
#define SFX_BAA_2 2
#define SFX_PLACE_DOG 3
#define SFX_BEAR_HORN 4
#define SFX_BEAR_GROWL 5

// Available streams
#define STREAM_BGM 0
#define STREAM_TOUCH_1 1
#define STREAM_SHEEP_1 2
#define STREAM_SHEEP_2 3
#define STREAM_BEAR_GROWL 4
#define STREAM_TOUCH_2 5
#define STREAM_HORN 6

#define STREAM_COUNT 7


/*
 * Initializes the audio system, opening all streams with default sounds
 */
void initialize_audio();

/*
 * Terminates the audio system, closing all streams
 */
void destroy_audio();

/*
 * Opens a sound effect or a track in a stream
 *  @param which The stream to use. Use one of the defined STREAM_ values
 *  @param sound The sound effect or track to use. Use one of the defined SFX_ or BGM_ values
 */
void stream_open( int which, int sound );

/*
 * Plays a stream
 *  @param which The stream to play. Use one of the defined STREAM_ values
 */
void stream_play( int which );

/*
 * Closes a stream
 *  @param which The stream to close. Use one of the defined STREAM_ values
 */
void stream_close( int which );

#endif /* AUDIO_H_ */
