/*
 * Copyright 2012 Sebastian Mihai
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * graphics.c
 *
 *  Created on: 2012-08-10
 *      Author: Sebastian Mihai
 */

#include <EGL/egl.h>
#include <GLES/gl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdlib.h>

#include "bbutil.h"
#include "defs.h"
#include "util.h"

float viewX, viewY;

float vertices[8] = {
		-1.0f, -1.0f,
		1.0f, -1.0f,
		-1.0f, 1.0f,
		1.0f, 1.0f,
	};

typedef struct {
	// texture coordinates
	GLfloat tc[8];
	// texture ID
	GLuint t;
} tex;

#define TEXTURE_SHEEP_1_1 0
#define TEXTURE_SHEEP_1_2 1
#define TEXTURE_SHEEP_2_1 2
#define TEXTURE_SHEEP_2_2 3
#define TEXTURE_SHEEP_3_1 4
#define TEXTURE_SHEEP_3_2 5
#define TEXTURE_PLAYFIELD 6
#define TEXTURE_BUSH_1 7
#define TEXTURE_BUSH_2 8
#define TEXTURE_BUSH_3 9
#define TEXTURE_BUSH_4 10
#define TEXTURE_BEAR_1 11
#define TEXTURE_BEAR_2 12
#define TEXTURE_TOUCH 13
#define TEXTURE_SCORE_D 14
#define TEXTURE_SCORE_C 15
#define TEXTURE_SCORE_B 16
#define TEXTURE_SCORE_A_MINUS 17
#define TEXTURE_SCORE_A 18
#define TEXTURE_SCORE_A_PLUS 19
#define TEXTURE_HORN 20
#define TEXTURE_MENU_TITLE 21
#define TEXTURE_MENU_SCORE 22
#define TEXTURE_MENU_CREDITS 23
#define TEXTURE_HORN_PARTIAL 24

tex sheepTextures[6];
tex playfieldTexture;
tex bushTextures[4];
tex bearTextures[2];
tex touchTexture;
tex scoreTextures[6];
tex hornTexture;
tex hornPartialTexture;
tex menuScoreTexture;
tex menuTitleTexture;
tex menuCreditsTexture;

int assignTexture( int type ) {
	if( type == ENTITY_SHEEP )
		return boundedRandom(3) * 2;

	if( type == ENTITY_OBSTACLE )
		return boundedRandom(4);

	if( type == ENTITY_BEAR )
		return boundedRandom(2);

	return TEXTURE_SHEEP_1_1;
}

int nextSheepTexture( int currentIndex ) {
	switch( currentIndex ) {
		case TEXTURE_SHEEP_1_1 : return TEXTURE_SHEEP_1_2;
		case TEXTURE_SHEEP_1_2 : return TEXTURE_SHEEP_1_1;
		case TEXTURE_SHEEP_2_1 : return TEXTURE_SHEEP_2_2;
		case TEXTURE_SHEEP_2_2 : return TEXTURE_SHEEP_2_1;
		case TEXTURE_SHEEP_3_1 : return TEXTURE_SHEEP_3_2;
		case TEXTURE_SHEEP_3_2 : return TEXTURE_SHEEP_3_1;
	}
	return TEXTURE_SHEEP_1_1;
}

int nextBearTexture( int currentIndex ) {
	switch( currentIndex ) {
		case 0 : return 1;
		case 1 : return 0;
	}
	return TEXTURE_SHEEP_1_1;
}

/*
 * Private helper function - get file paths for textures
 */
char* get_filename_for_texture( int which ) {

	switch( which ) {
	case TEXTURE_SHEEP_1_1 : return "app/native/assets/textures/sheep_1_1.png";
	case TEXTURE_SHEEP_1_2 : return "app/native/assets/textures/sheep_1_2.png";
	case TEXTURE_SHEEP_2_1 : return "app/native/assets/textures/sheep_2_1.png";
	case TEXTURE_SHEEP_2_2 : return "app/native/assets/textures/sheep_2_2.png";
	case TEXTURE_SHEEP_3_1 : return "app/native/assets/textures/sheep_3_1.png";
	case TEXTURE_SHEEP_3_2 : return "app/native/assets/textures/sheep_3_2.png";
	case TEXTURE_PLAYFIELD : return "app/native/assets/textures/playfield.png";
	case TEXTURE_BUSH_1 : return "app/native/assets/textures/bush.png";
	case TEXTURE_BUSH_2 : return "app/native/assets/textures/hay.png";
	case TEXTURE_BUSH_3 : return "app/native/assets/textures/water.png";
	case TEXTURE_BUSH_4 : return "app/native/assets/textures/logs.png";
	case TEXTURE_BEAR_1 : return "app/native/assets/textures/bear_1.png";
	case TEXTURE_BEAR_2 : return "app/native/assets/textures/bear_2.png";
	case TEXTURE_TOUCH : return "app/native/assets/textures/touch.png";
	case TEXTURE_SCORE_D : return "app/native/assets/textures/d.png";
	case TEXTURE_SCORE_C : return "app/native/assets/textures/c.png";
	case TEXTURE_SCORE_B : return "app/native/assets/textures/b.png";
	case TEXTURE_SCORE_A_MINUS : return "app/native/assets/textures/aminus.png";
	case TEXTURE_SCORE_A : return "app/native/assets/textures/a.png";
	case TEXTURE_SCORE_A_PLUS : return "app/native/assets/textures/aplus.png";

	case TEXTURE_HORN : return "app/native/assets/textures/horn.png";
	case TEXTURE_HORN_PARTIAL : return "app/native/assets/textures/hornpartial.png";

	case TEXTURE_MENU_TITLE : return "app/native/assets/textures/title.png";
	case TEXTURE_MENU_CREDITS : return "app/native/assets/textures/credits.png";
	case TEXTURE_MENU_SCORE : return "app/native/assets/textures/score.png";
	}

	return TEXTURE_SHEEP_1_1;
}

void render_begin() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_TEXTURE_2D);
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
}

void render_end() {
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisable(GL_TEXTURE_2D);
}

void render_swap() {
	bbutil_swap();
}

void render_quad( tex *t, float scaleX, float scaleY, float transX, float transY ) {
	glPushMatrix();
	glTranslatef( transX, transY, 0.0f );
	glScalef( scaleX, scaleY, 1.0f );
	glVertexPointer(2, GL_FLOAT, 0, vertices);
	glTexCoordPointer(2, GL_FLOAT, 0, t->tc);
	glBindTexture(GL_TEXTURE_2D, t->t);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	glPopMatrix();
}

void render_entity( entity *e ) {
	int supportedType = 0;
	glPushMatrix();
	glTranslatef( e->x, e->y, 0.0f );
	glScalef( e->radius, e->radius, 1.0f );
	glMatrixMode(GL_MODELVIEW);
	glRotatef( e->angle, 0.0f, 0.0f, 1.0f);
	glVertexPointer(2, GL_FLOAT, 0, vertices);

	if( e->type == ENTITY_SHEEP ) {
		glTexCoordPointer(2, GL_FLOAT, 0, sheepTextures[e->textureIndex].tc);
		glBindTexture(GL_TEXTURE_2D, sheepTextures[e->textureIndex].t);
		supportedType = 1;
	} else if( e->type == ENTITY_OBSTACLE ) {
		glTexCoordPointer(2, GL_FLOAT, 0, bushTextures[e->textureIndex].tc);
		glBindTexture(GL_TEXTURE_2D, bushTextures[e->textureIndex].t);
		supportedType = 1;
	} else if( e->type == ENTITY_BEAR ) {
		glTexCoordPointer(2, GL_FLOAT, 0, bearTextures[e->textureIndex].tc);
		glBindTexture(GL_TEXTURE_2D, bearTextures[e->textureIndex].t);
		supportedType = 1;
	}

	if( supportedType ) {
		glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	}

	glPopMatrix();
}

void render_save_area_of_effect( float x, float y, float radius ) {
	render_quad( &touchTexture, radius, radius, x, y );
}

void render_playfield() {

	int x,y;
	int w,h;

	w=80;
	h=80;

	for( x=-5*w; x<PLAYFIELD_WIDTH+5*w; x+=w  )
		for( y=-5*h; y<PLAYFIELD_HEIGHT+5*h; y+=h  ) {
			render_quad( &playfieldTexture, (float)w, (float)h, (float)(x + w/2 - PLAYFIELD_WIDTH/2 ), (float)(y + h/2 - PLAYFIELD_HEIGHT /2 ) );
		}

	w=40;
	h=40;

	for( x=-w; x<PLAYFIELD_WIDTH+w; x+=w  ) {
		render_quad( &bushTextures[1], (float)w, (float)h, (float)(x + w/2 - PLAYFIELD_WIDTH/2 ), (float)(-h - PLAYFIELD_HEIGHT /2 ) );
		render_quad( &bushTextures[1], (float)w, (float)h, (float)(x + w/2 - PLAYFIELD_WIDTH/2 ), (float)(PLAYFIELD_HEIGHT + h - PLAYFIELD_HEIGHT /2 ) );
	}
	for( y=-h; y<PLAYFIELD_HEIGHT+h; y+=h  ) {
		render_quad( &bushTextures[1], (float)w, (float)h, (float)(-w - PLAYFIELD_WIDTH/2 ), (float)(y + h/2 - PLAYFIELD_HEIGHT /2 ) );
		render_quad( &bushTextures[1], (float)w, (float)h, (float)(PLAYFIELD_WIDTH+w - PLAYFIELD_WIDTH/2 ), (float)(y + h/2 - PLAYFIELD_HEIGHT /2 ) );
	}
}


void load_texture( tex *t, int which ) {
	float tex_x = 1.0f, tex_y = 1.0f;

	bbutil_load_texture( get_filename_for_texture( which ), NULL, NULL, &tex_x, &tex_y, &t->t );
	t->tc[0] = 0.0f;
	t->tc[1] = 0.0f;
	t->tc[2] = tex_x;
	t->tc[3] = 0.0f;
	t->tc[4] = 0.0f;
	t->tc[5] = tex_y;
	t->tc[6] = tex_x;
	t->tc[7] = tex_y;
}

void load_textures() {
	int i;

	for( i=0; i<6; i++) {
		load_texture( &sheepTextures[i], i );
	}

	load_texture( &playfieldTexture, TEXTURE_PLAYFIELD );
	load_texture( &bushTextures[0], TEXTURE_BUSH_1 );
	load_texture( &bushTextures[1], TEXTURE_BUSH_2 );
	load_texture( &bushTextures[2], TEXTURE_BUSH_3 );
	load_texture( &bushTextures[3], TEXTURE_BUSH_4 );

	load_texture( &bearTextures[0], TEXTURE_BEAR_1 );
	load_texture( &bearTextures[1], TEXTURE_BEAR_2 );
	load_texture( &touchTexture, TEXTURE_TOUCH );

	load_texture( &scoreTextures[0], TEXTURE_SCORE_D );
	load_texture( &scoreTextures[1], TEXTURE_SCORE_C );
	load_texture( &scoreTextures[2], TEXTURE_SCORE_B );
	load_texture( &scoreTextures[3], TEXTURE_SCORE_A_MINUS );
	load_texture( &scoreTextures[4], TEXTURE_SCORE_A );
	load_texture( &scoreTextures[5], TEXTURE_SCORE_A_PLUS );

	load_texture( &hornTexture, TEXTURE_HORN );
	load_texture( &hornPartialTexture, TEXTURE_HORN_PARTIAL );

	load_texture( &menuTitleTexture, TEXTURE_MENU_TITLE );
	load_texture( &menuScoreTexture, TEXTURE_MENU_SCORE );
	load_texture( &menuCreditsTexture, TEXTURE_MENU_CREDITS );
}

void initialize_graphics( int *surfaceW, int *surfaceH ) {

	int surfaceWidth, surfaceHeight;

	viewX = 0.0f;
	viewY = 0.0f;

	eglQuerySurface(egl_disp, egl_surf, EGL_WIDTH, &surfaceWidth);
	eglQuerySurface(egl_disp, egl_surf, EGL_HEIGHT, &surfaceHeight);

	EGLint err = eglGetError();
	if (err != 0x3000) {
		exit(1);
	}

	glShadeModel(GL_SMOOTH);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_TEXTURE_2D);

	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	glViewport(0, 0, surfaceWidth, surfaceHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrthof(0.0f, (float) (surfaceWidth) / (float) (surfaceHeight), 0.0f,
			1.0f, -1.0f, 1.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glScalef(1.0f / surfaceHeight, 1.0f / surfaceHeight, 1.0f);

	glTranslatef((float)(surfaceWidth/2), (float)(surfaceHeight/2) , 0.0f);

	*surfaceW = surfaceWidth;
	*surfaceH = surfaceHeight;
}

void scroll_screen( float x, float y ) {
	viewX+=x;
	viewY+=y;
	glTranslatef( x, y, 0.0f);
}

void render_menu() {
	render_quad( &menuTitleTexture, 200.0f, 100.0f, 0.0f, 100.0f );
	render_quad( &menuCreditsTexture, 150.0f, 40.0f, 400.0f, -270.0f );
}

void render_powerup() {
	float scale = 40.0f;
	render_quad( &hornTexture, scale, scale, -viewX-465.0f, -viewY+250.0f );
}

void render_powerup_partial( float percentage ) {
	float scale = 40.0f;
	render_quad( &hornPartialTexture, scale, scale, -viewX-465.0f, -viewY+250.0f );
	render_quad( &hornTexture, percentage*scale, percentage*scale, -viewX-465.0f, -viewY+250.0f );
}

void render_score( int score ) {
	render_quad( &menuScoreTexture, 128.0f, 24.0f, -40.0f, -150.0f );
	render_quad( &scoreTextures[score], 40.0f, 40.0f, 115.0f, -150.0f );
}
