/*
 * Copyright (c) 2011-2012 Research In Motion Limited.
 *
 * Modified by Sebastian Mihai to interact with the manager module.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <bps/navigator.h>
#include <bps/screen.h>
#include <stdlib.h>
#include <assert.h>

#include "bbutil.h"
#include "manager.h"

int main(int argc, char *argv[]) {
    int rc;
    int exit_application = 0;
    static screen_context_t screen_cxt;

    //Create a screen context that will be used to create an EGL surface to to receive libscreen events
    screen_create_context(&screen_cxt, 0);

    //Initialize BPS library
    bps_initialize();

    //Use utility code to initialize EGL for rendering with GL ES 1.1
    if (EXIT_SUCCESS != bbutil_init_egl(screen_cxt)) {
        fprintf(stderr, "bbutil_init_egl failed\n");
        bbutil_terminate();
        screen_destroy_context(screen_cxt);
        return 0;
    }

    //Initialize application logic
    if (EXIT_SUCCESS != initialize_game_manager()) {
        fprintf(stderr, "initialize failed\n");
        bbutil_terminate();
        screen_destroy_context(screen_cxt);
        return 0;
    }

    //Signal BPS library that navigator and screen events will be requested
    if (BPS_SUCCESS != screen_request_events(screen_cxt)) {
        fprintf(stderr, "screen_request_events failed\n");
        bbutil_terminate();
        screen_destroy_context(screen_cxt);
        return 0;
    }

    if (BPS_SUCCESS != navigator_request_events(0)) {
        fprintf(stderr, "navigator_request_events failed\n");
        bbutil_terminate();
        screen_destroy_context(screen_cxt);
        return 0;
    }

    //Signal BPS library that navigator orientation is not to be locked
    if (BPS_SUCCESS != navigator_rotation_lock(false)) {
        fprintf(stderr, "navigator_rotation_lock failed\n");
        bbutil_terminate();
        screen_destroy_context(screen_cxt);
        return 0;
    }

    while ( !exit_application ) {
        //Request and process all available BPS events
        bps_event_t *event = NULL;

        while( 1 ) {
            rc = bps_get_event(&event, 0);
            assert( BPS_SUCCESS == rc );

            if( event ) {
                int bpsEventDomain = bps_event_get_domain( event );

                if ( bpsEventDomain == screen_get_domain() ) {
                	manager_handle_screen_event( event );
                } else if ( ( navigator_get_domain() == bpsEventDomain ) && ( bps_event_get_code(event) == NAVIGATOR_EXIT ) ) {
                    exit_application = 1;
                }
            } else {
                break;
            }
        }

        manager_run_frame();
    }

    manager_terminate();
    screen_stop_events(screen_cxt);
    bps_shutdown();
    bbutil_terminate();
    screen_destroy_context(screen_cxt);
    return 0;
}
